use crate::prelude::*;

use anyhow::Result;

#[derive(Debug)]
pub struct RenderPass<'a> {
    device: &'a Device,
    render_pass: VkRenderPass,
}

impl<'a> RenderPass<'a> {
    pub fn new(
        device: &'a Device,
        sub_passes: &[VkSubpassDescription],
        attachments: &[VkAttachmentDescription],
        dependencies: &[VkSubpassDependency],
    ) -> Result<RenderPass<'a>> {
        let render_pass_ci = VkRenderPassCreateInfo::new(
            VK_RENDERPASS_CREATE_NULL_BIT,
            attachments,
            sub_passes,
            dependencies,
        );

        let render_pass = device.create_render_pass(&render_pass_ci)?;

        Ok(RenderPass {
            device,
            render_pass,
        })
    }
}

impl<'a> VulkanDevice for RenderPass<'a> {
    fn device(&self) -> &Device {
        &self.device
    }
}

impl_vk_handle!(RenderPass<'a>, VkRenderPass, render_pass);

impl<'a> Drop for RenderPass<'a> {
    fn drop(&mut self) {
        self.device.destroy_render_pass(self.render_pass);
    }
}
