use crate::prelude::*;

use anyhow::Result;

#[derive(Debug)]
pub struct Semaphore<'a> {
    device: &'a Device,
    semaphore: VkSemaphore,
}

impl<'a> Semaphore<'a> {
    pub fn new(device: &'a Device) -> Result<Semaphore<'a>> {
        let semaphore_ci = VkSemaphoreCreateInfo::new(VK_SEMAPHORE_CREATE_NULL_BIT);

        let semaphore = device.create_semaphore(&semaphore_ci)?;

        Ok(Semaphore { device, semaphore })
    }
}

impl<'a> VulkanDevice for Semaphore<'a> {
    fn device(&self) -> &Device {
        &self.device
    }
}

impl_vk_handle!(Semaphore<'a>, VkSemaphore, semaphore);

impl<'a> Drop for Semaphore<'a> {
    fn drop(&mut self) {
        self.device.destroy_semaphore(self.semaphore);
    }
}
